@@ -15,11 +15,18 @@ fit_hbd_psr_on_grid = function(	tree,
 								splines_degree		= 1,		# integer, either 1 or 2 or 3, specifying the degree for the splines defined by the PSR on the age grid.
 								condition			= "stem",	# one of "crown" or "stem", specifying whether to condition the likelihood on the survival of the stem group or the crown group. It is recommended to use "stem" when oldest_age>root_age, and "crown" when oldest_age==root_age. This argument is similar to the "cond" argument in the R function RPANDA::likelihood_bd. Note that "crown" really only makes sense when oldest_age==root_age.
 								relative_dt			= 1e-3,		# maximum relative time step allowed for integration. Smaller values increase the accuracy of the computed likelihoods, but increase computation time. Typical values are 0.0001-0.001. The default is usually sufficient.
-								Ntrials				= 1,
-								Nthreads			= 1,
-								max_model_runtime	= NULL,		# maximum time (in seconds) to allocate for each likelihood evaluation. Use this to escape from badly parameterized models during fitting (this will likely cause the affected fitting trial to fail). If NULL or <=0, this option is ignored.
-								fit_control			= list()){	# a named list containing options for the nlminb fitting routine (e.g. iter.max and rel.tol)
+								Ntrials					= 1,
+								Nbootstraps				= 0,		# (integer) optional number of parametric-bootstrap samples (random trees generated using the fitted PSR) for estimating confidence intervals of fitted parameters. If 0, no parametric bootstrapping is performed. Typical values are 10-100.
+								Ntrials_per_bootstrap	= NULL,		# (integer) optional number of fitting trials for each bootstrap sampling. If NULL, this is set equal to Ntrials. A smaller Ntrials_per_bootstrap will reduce computation, at the expense of increasing the estimated confidence intervals (i.e. yielding more conservative estimates of confidence).
+								Nthreads				= 1,
+								max_model_runtime		= NULL,		# maximum time (in seconds) to allocate for each likelihood evaluation. Use this to escape from badly parameterized models during fitting (this will likely cause the affected fitting trial to fail). If NULL or <=0, this option is ignored.
+								fit_control				= list(),	# a named list containing options for the nlminb fitting routine (e.g. iter.max and rel.tol)
+								verbose					= FALSE,		# boolean, specifying whether to print informative messages
+								verbose_prefix			= ""){			# string, specifying the line prefix when printing messages. Only relevant if verbose==TRUE.
 	# basic error checking
+	if(verbose) cat(sprintf("%sChecking input variables..\n",verbose_prefix))
+	original_Ntips = length(tree$tip.label)
+	if((Nbootstraps>0) && (!is.null(Ntrials_per_bootstrap)) && (Ntrials_per_bootstrap<=0)) stop(sprintf("ERROR: Ntrials_per_bootstrap must be strictly positive, if bootstrapping is requested"))
 	if(tree$Nnode<2) return(list(success = FALSE, error="Input tree is too small"));
 	if(age0<0) return(list(success = FALSE, error="age0 must be non-negative"));
 	root_age = get_tree_span(tree)$max_distance
@@ -29,6 +36,7 @@ fit_hbd_psr_on_grid = function(	tree,
 
 	# trim tree at age0 if needed, while shifting time for the subsequent analyses (i.e. new ages will start counting at age0)
 	if(age0>0){
+		if(verbose) cat(sprintf("%sTrimming tree at age0=%g..\n",verbose_prefix,age0))
 		tree = trim_tree_at_height(tree,height=root_age-age0)$tree
 		if(tree$Nnode<2) return(list(success = FALSE, error=sprintf("Tree is too small after trimming at age0 (%g)",age0)));
 		if(!is.null(oldest_age)) oldest_age	= oldest_age - age0	
@@ -37,6 +45,7 @@ fit_hbd_psr_on_grid = function(	tree,
 	}
 								
 	# pre-compute some tree stats
+	if(verbose) cat(sprintf("%sPrecomputing some stats about the tree..\n",verbose_prefix))
 	LTT0				= length(tree$tip.label);
 	lineage_counter 	= count_lineages_through_time(tree, Ntimes=log2(LTT0), include_slopes=TRUE);
 	sorted_node_ages	= sort(get_all_branching_ages(tree));
@@ -61,6 +70,7 @@ fit_hbd_psr_on_grid = function(	tree,
 	if(NG==1) splines_degree = 1; # no point in using splines since PSR is assumed to be time-independent
 	
 	# reformat shape of input params to an internally standardized format
+	if(verbose) cat(sprintf("%sPreparing for fitting..\n",verbose_prefix))
 	if(length(min_PSR)==1) min_PSR = rep(min_PSR,times=NG);
 	if(length(max_PSR)==1) max_PSR = rep(max_PSR,times=NG);
 	min_PSR = pmax(0,min_PSR);
@@ -163,6 +173,7 @@ fit_hbd_psr_on_grid = function(	tree,
 
 	# run one or more independent fitting trials
     if((Ntrials>1) && (Nthreads>1) && (.Platform$OS.type!="windows")){
+		if(verbose) cat(sprintf("%sFitting %d model parameters (%d trials, parallelized)..\n",verbose_prefix,NFP,Ntrials))
 		# run trials in parallel using multiple forks
 		# Note: Forks (and hence shared memory) are not available on Windows
 		fits = parallel::mclapply(	1:Ntrials, 
@@ -172,6 +183,7 @@ fit_hbd_psr_on_grid = function(	tree,
 									mc.cleanup = TRUE);
 	}else{
 		# run in serial mode
+		if(verbose) cat(sprintf("%sFitting %d model parameters (%s)..\n",verbose_prefix,NFP,(if(Ntrials==1) "1 trial" else sprintf("%d trials",Ntrials))))
 		fits = sapply(1:Ntrials,function(x) NULL)
 		for(trial in 1:Ntrials){
 			fits[[trial]] = fit_single_trial(trial)
@@ -190,10 +202,82 @@ fit_hbd_psr_on_grid = function(	tree,
 	fitted_PSR			= fitted_param_values[1:NG]
 
 	# reverse any time shift due to earlier tree trimming
-	age_grid = age_grid + age0
+	age_grid 	= age_grid + age0
+	oldest_age 	= oldest_age + age0
+	root_age 	= root_age + age0
 	
 	# calculate deterministic LTT of fitted congruence class on the age grid
-	fitted_LTT = LTT0 * exp(-get_antiderivative_of_piecewise_linear_function(age_grid, age0, fitted_PSR, splines_degree, age_grid));
+	fitted_LTT = LTT0 * exp(-get_antiderivative_of_splines_function(age_grid, age0, fitted_PSR, splines_degree, age_grid));
+	
+	
+	#######################################################################
+	# estimate confidence intervals if needed, via parametric bootstrapping
+	
+	if(Nbootstraps>0){
+		if(verbose) cat(sprintf("%sEstimating confidence intervals using %d parametric bootstraps..\n",verbose_prefix,Nbootstraps))
+		if(is.null(Ntrials_per_bootstrap)) Ntrials_per_bootstrap = max(1,Ntrials)
+		#   include a dummy age grid point at the end of age_grid if needed (extrapolating the fitted PSR as a constant), so as to cover the root age
+		#   also include a dummy age grid point at the beginning if necessary (extrapolating PSR as a constant), to cover present-day (age 0)
+		sim_age_grid = age_grid
+		sim_PSR		 = fitted_PSR
+		if(tail(sim_age_grid,1)<root_age){
+			# extrapolate sim_PSR (as a constant) all the way to the root (a bit more to avoid rounding errors)
+			sim_age_grid = c(sim_age_grid, root_age*1.01)
+			sim_PSR 	 = c(sim_PSR, tail(sim_PSR,1));
+		}
+		if(sim_age_grid[1]>0){
+			# extrapolate sim_PSR (as a constant) all the way to the present-day (age 0)
+			sim_age_grid = c(0,sim_age_grid)
+			sim_PSR		 = c(sim_PSR[1],sim_PSR)
+		}
+		bootstrap_PSR = matrix(NA,nrow=Nbootstraps,ncol=NG)
+		NBsucceeded	  = 0
+		for(b in 1:Nbootstraps){
+			# simulate model with fitted parameters
+			if(verbose) cat(sprintf("%s  Bootstrap #%d..\n",verbose_prefix,b))
+			bootstrap = castor::generate_tree_hbd_reverse(	Ntips			= original_Ntips,
+																crown_age		= root_age,
+																age_grid		= sim_age_grid, 
+																PSR				= sim_PSR,
+																splines_degree	= splines_degree,
+																relative_dt		= relative_dt)
+			if(!bootstrap$success) return(list(success=FALSE, error=sprintf("Bootstrapping failed: Could not generate tree for the fitted PSR: %s",bootstrap$error), age_grid=age_grid, fitted_PSR=fitted_PSR, loglikelihood=loglikelihood));
+			bootstrap_tree = bootstrap$trees[[1]]
+
+			# fit PSR to simulated tree
+			fit = fit_hbd_psr_on_grid(	bootstrap_tree, 
+										oldest_age	= oldest_age,
+										age0		= age0,
+										age_grid	= age_grid,
+										min_PSR		= min_PSR,
+										max_PSR		= max_PSR,
+										guess_PSR	= guess_PSR,
+										fixed_PSR	= fixed_PSR,
+										splines_degree		= splines_degree,
+										condition			= condition,
+										relative_dt			= relative_dt,
+										Ntrials				= Ntrials_per_bootstrap,
+										Nbootstraps			= 0,
+										Nthreads			= 1,
+										max_model_runtime	= max_model_runtime,
+										fit_control			= fit_control,
+										verbose				= verbose,
+										verbose_prefix		= paste0(verbose_prefix,"    "))
+			if(!fit$success){
+				if(verbose) cat(sprintf("%s  WARNING: Fitting failed for this bootstrap: %s\n",verbose_prefix,fit$error))
+			}else{
+				bootstrap_PSR[b,] = fit$fitted_PSR
+				NBsucceeded = NBsucceeded + 1
+			}
+		}
+		# calculate standard errors and confidence intervals from distribution of bootstrapped parameters
+		standard_errors = sqrt(pmax(0, colMeans(bootstrap_PSR^2, na.rm=TRUE) - colMeans(bootstrap_PSR, na.rm=TRUE)^2))
+		quantiles = sapply(1:ncol(bootstrap_PSR), FUN=function(p) quantile(bootstrap_PSR[,p], probs=c(0.25, 0.75, 0.025, 0.975), na.rm=TRUE, type=8))
+		CI50lower = quantiles[1,]
+		CI50upper = quantiles[2,]
+		CI95lower = quantiles[3,]
+		CI95upper = quantiles[4,]
+	}
 		
 	# return results
 	return(list(success					= TRUE,
@@ -209,7 +293,13 @@ fit_hbd_psr_on_grid = function(	tree,
 				BIC						= log(sum((sorted_node_ages<=oldest_age) & (sorted_node_ages>=age0)))*NFP - 2*loglikelihood,
 				converged				= fits[[best]]$converged,
 				Niterations				= fits[[best]]$Niterations,
-				Nevaluations			= fits[[best]]$Nevaluations));
+				Nevaluations			= fits[[best]]$Nevaluations,
+				standard_errors			= (if(Nbootstraps>0) standard_errors else NULL),
+				CI50lower				= (if(Nbootstraps>0) CI50lower else NULL),
+				CI50upper				= (if(Nbootstraps>0) CI50upper else NULL),
+				CI95lower				= (if(Nbootstraps>0) CI95lower else NULL),
+				CI95upper				= (if(Nbootstraps>0) CI95upper else NULL)))
+
 }
 
 
